@@ -61,7 +61,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		QStandardItemModel *sourceModel;
 		QTableView *completionView;
 		QMenu *cmenu;
-		QAction *undoAction, *redoAction, *cutAction, *copyAction, *pasteAction, *deleteAction, *selectAllAction, *clearAction;
+		QAction *undoAction, *redoAction, *cutAction, *copyAction, *pasteAction, *deleteAction, *selectAllAction, *clearAction, *statusOffAction, *statusDelayAction, *statusNoDelayAction;
 		QTimer *completionTimer, *toolTipTimer;
 		ExpressionTipLabel *tipLabel;
 		QWidget *tb;
@@ -104,7 +104,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		std::string expression() const;
 		QSize sizeHint() const override;
 
-		void wrapSelection(const QString &text = QString(), bool insert_before = false, bool add_parentheses = false, bool add_comma = false);
+		void wrapSelection(const QString &text = QString(), bool insert_before = false, bool add_parentheses = false, bool add_comma = false, const QString &add_arg = QString(), bool quote = false);
 		bool doChainMode(const QString &op);
 		bool expressionHasChanged();
 		void setExpressionHasChanged(bool);
@@ -145,6 +145,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		void editDelete();
 		void insertDate();
 		void insertMatrix();
+		void completeOrActivateFirst();
 		bool complete(MathStructure* = NULL, const QPoint& = QPoint(), bool = false);
 
 	signals:
