@@ -19,6 +19,7 @@
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
 #include <QPushButton>
+#include <QCheckBox>
 #include <QLineEdit>
 #include <QKeyEvent>
 #include <QApplication>
@@ -28,8 +29,9 @@
 #include "functionsdialog.h"
 #include "itemproxymodel.h"
 #include "functioneditdialog.h"
+#include "dataseteditdialog.h"
 
-FunctionsDialog::FunctionsDialog(QWidget *parent) : QDialog(parent) {
+FunctionsDialog::FunctionsDialog(QWidget *parent) : QDialog(parent, Qt::Window) {
 	QVBoxLayout *topbox = new QVBoxLayout(this);
 	setWindowTitle(tr("Functions"));
 	QHBoxLayout *hbox = new QHBoxLayout();
@@ -64,7 +66,9 @@ FunctionsDialog::FunctionsDialog(QWidget *parent) : QDialog(parent) {
 	searchEdit = new QLineEdit(this);
 	searchEdit->addAction(LOAD_ICON("edit-find"), QLineEdit::LeadingPosition);
 #ifdef _WIN32
+#	if (QT_VERSION < QT_VERSION_CHECK(6, 2, 0))
 	searchEdit->setTextMargins(22, 0, 0, 0);
+#	endif
 #endif
 	searchEdit->installEventFilter(this);
 	vbox->addWidget(searchEdit, 0);
@@ -87,6 +91,8 @@ FunctionsDialog::FunctionsDialog(QWidget *parent) : QDialog(parent) {
 	calculateButton->setDefault(true);
 	applyButton = new QPushButton(tr("Apply"), this); box->addWidget(applyButton); connect(applyButton, SIGNAL(clicked()), this, SLOT(applyClicked()));
 	insertButton = new QPushButton(tr("Insert"), this); box->addWidget(insertButton); connect(insertButton, SIGNAL(clicked()), this, SLOT(insertClicked()));
+	box->addSpacing(24);
+	favouriteButton = new QCheckBox(tr("Favorite"), this); box->addWidget(favouriteButton); connect(favouriteButton, SIGNAL(clicked()), this, SLOT(favouriteClicked()));
 	box->addStretch(1);
 	hbox->addLayout(box, 0);
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Close, Qt::Horizontal, this);
@@ -94,13 +100,11 @@ FunctionsDialog::FunctionsDialog(QWidget *parent) : QDialog(parent) {
 	selected_category = "All";
 	updateFunctions();
 	functionsView->setFocus();
-	functionsModel->setFilter("All");
 	connect(searchEdit, SIGNAL(textChanged(const QString&)), this, SLOT(searchChanged(const QString&)));
 	connect(buttonBox->button(QDialogButtonBox::Close), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(categoriesView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(selectedCategoryChanged(QTreeWidgetItem*, QTreeWidgetItem*)));
 	connect(functionsView->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(selectedFunctionChanged(const QModelIndex&, const QModelIndex&)));
 	connect(functionsView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(calculateClicked()));
-	selectedFunctionChanged(QModelIndex(), QModelIndex());
 	if(!settings->functions_geometry.isEmpty()) restoreGeometry(settings->functions_geometry);
 	else resize(900, 800);
 	if(!settings->functions_vsplitter_state.isEmpty()) vsplitter->restoreState(settings->functions_vsplitter_state);
@@ -172,14 +176,76 @@ void FunctionsDialog::newClicked() {
 			QModelIndexList list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
 			if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
 		} else if(replaced_item && !replaced_item->isActive()) {
-			ADD_INACTIVE_CATEGORY
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
 		}
 		selected_item = f;
-		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true)));
+		if(f->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(f->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(f->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateFunctions();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) functionsView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) f), Qt::UserRole);
 		sourceModel->appendRow(item);
-		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + f->category()) {
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + f->category() && (selected_category != "Uncategorized" || !f->category().empty())) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(!list.isEmpty()) {
+				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			}
+		} else {
+			functionsModel->invalidate();
+		}
+		sourceModel->sort(0);
+		QModelIndex index = functionsModel->mapFromSource(item->index());
+		if(index.isValid()) {
+			functionsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			functionsView->scrollTo(index);
+		}
+		if(f != replaced_item && !f->isHidden()) settings->favourite_functions.push_back(f);
+		emit itemsChanged();
+	}
+}
+void FunctionsDialog::editClicked() {
+	QModelIndex index = functionsView->selectionModel()->currentIndex();
+	if(!index.isValid()) return;
+	MathFunction *replaced_item = NULL;
+	MathFunction *f = (MathFunction*) index.data(Qt::UserRole).value<void*>();
+	if(f && ((f->subtype() != SUBTYPE_DATA_SET && FunctionEditDialog::editFunction(this, f, &replaced_item)) || (f->subtype() == SUBTYPE_DATA_SET && DataSetEditDialog::editDataset(this, (DataSet*) f, &replaced_item)))) {
+		sourceModel->removeRow(functionsModel->mapToSource(functionsView->selectionModel()->currentIndex()).row());
+		if(replaced_item && !CALCULATOR->hasFunction(replaced_item)) {
+			QModelIndexList list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
+			if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
+		} else if(replaced_item && !replaced_item->isActive()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Inactive", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Inactive"); l << "Inactive"; new QTreeWidgetItem(categoriesView, l);}
+		}
+		selected_item = f;
+		if(f->category().empty()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems("Uncategorized", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {QStringList l; l << tr("Uncategorized"); l << "Uncategorized"; new QTreeWidgetItem(categoriesView->topLevelItem(2), l);}
+		} else if(f->category() != CALCULATOR->temporaryCategory()) {
+			QList<QTreeWidgetItem*> list = categoriesView->findItems(QString("/") + QString::fromStdString(f->category()), Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+			if(list.isEmpty()) {
+				if(selected_category != "All") selected_category = "User items";
+				updateFunctions();
+				emit itemsChanged();
+				return;
+			}
+		}
+		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) functionsView)));
+		item->setEditable(false);
+		item->setData(QVariant::fromValue((void*) f), Qt::UserRole);
+		sourceModel->appendRow(item);
+		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + f->category() && (selected_category != "Uncategorized" || !f->category().empty())) {
 			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
 			if(!list.isEmpty()) {
 				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
@@ -196,40 +262,33 @@ void FunctionsDialog::newClicked() {
 		emit itemsChanged();
 	}
 }
-void FunctionsDialog::editClicked() {
+void FunctionsDialog::favouriteClicked() {
 	QModelIndex index = functionsView->selectionModel()->currentIndex();
 	if(!index.isValid()) return;
-	MathFunction *replaced_item = NULL;
 	MathFunction *f = (MathFunction*) index.data(Qt::UserRole).value<void*>();
-	if(f && FunctionEditDialog::editFunction(this, f, &replaced_item)) {
-		sourceModel->removeRow(functionsModel->mapToSource(functionsView->selectionModel()->currentIndex()).row());
-		if(replaced_item && !CALCULATOR->hasFunction(replaced_item)) {
-			QModelIndexList list = sourceModel->match(sourceModel->index(0, 0), Qt::UserRole, QVariant::fromValue((void*) replaced_item), 1, Qt::MatchExactly);
-			if(!list.isEmpty()) sourceModel->removeRow(list[0].row());
-		} else if(replaced_item && !replaced_item->isActive()) {
-			ADD_INACTIVE_CATEGORY
-		}
-		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true)));
-		item->setEditable(false);
-		item->setData(QVariant::fromValue((void*) f), Qt::UserRole);
-		sourceModel->appendRow(item);
-		selected_item = f;
-		if(selected_category != "All" && selected_category != "User items" && selected_category != std::string("/") + f->category()) {
-			QList<QTreeWidgetItem*> list = categoriesView->findItems("User items", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
-			if(!list.isEmpty()) {
-				categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+	if(!f) return;
+	if(favouriteButton->isChecked()) {
+		settings->favourite_functions.push_back(f);
+	} else {
+		for(size_t i = 0; i < settings->favourite_functions.size(); i++) {
+			if(settings->favourite_functions[i] == f) {
+				settings->favourite_functions.erase(settings->favourite_functions.begin() + i);
+				break;
 			}
-		} else {
-			functionsModel->invalidate();
 		}
-		sourceModel->sort(0);
-		QModelIndex index = functionsModel->mapFromSource(item->index());
-		if(index.isValid()) {
-			functionsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
-			functionsView->scrollTo(index);
+		if(selected_category == "Favorites") {
+			if(settings->favourite_functions.empty()) {
+				QList<QTreeWidgetItem*> list = categoriesView->findItems("All", Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap, 1);
+				if(!list.isEmpty()) categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			} else {
+				selected_item = NULL;
+				functionsModel->invalidate();
+				selectedFunctionChanged(functionsView->selectionModel()->currentIndex(), functionsView->selectionModel()->currentIndex());
+			}
 		}
-		emit itemsChanged();
 	}
+	settings->favourite_functions_changed = true;
+	emit itemsChanged();
 }
 void FunctionsDialog::delClicked() {
 	QModelIndex index = functionsView->selectionModel()->currentIndex();
@@ -274,8 +333,8 @@ void FunctionsDialog::deactivateClicked() {
 		if(!list.isEmpty()) {
 			categoriesView->setCurrentItem(list[0], 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 		} else if(!f->isActive()) {
-				QStringList l; l << tr("Inactive"); l << "Inactive";
-				categoriesView->setCurrentItem(new QTreeWidgetItem(categoriesView, l), 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+			QStringList l; l << tr("Inactive"); l << "Inactive";
+			categoriesView->setCurrentItem(new QTreeWidgetItem(categoriesView, l), 0, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
 		}
 		emit itemsChanged();
 	}
@@ -378,7 +437,11 @@ void FunctionsDialog::selectedFunctionChanged(const QModelIndex &index, const QM
 							str += tr("default:", "argument default").toStdString();
 							str += " ";
 							ParseOptions pa = settings->evalops.parse_options; pa.base = 10;
-							str += CALCULATOR->localizeExpression(f->getDefaultValue(i2), pa);
+							std::string str2 = CALCULATOR->localizeExpression(f->getDefaultValue(i2), pa);
+							gsub("*", settings->multiplicationSign(), str2);
+							gsub("/", settings->divisionSign(false), str2);
+							gsub("-", SIGN_MINUS, str2);
+							str += str2;
 						}
 						str += ")";
 					}
@@ -437,6 +500,14 @@ void FunctionsDialog::selectedFunctionChanged(const QModelIndex &index, const QM
 			insertButton->setEnabled(f->isActive());
 			calculateButton->setEnabled(f->isActive());
 			delButton->setEnabled(f->isLocal());
+			favouriteButton->setEnabled(true);
+			favouriteButton->setChecked(false);
+			for(size_t i = 0; i < settings->favourite_functions.size(); i++) {
+				if(settings->favourite_functions[i] == f) {
+					favouriteButton->setChecked(true);
+					break;
+				}
+			}
 			deactivateButton->setEnabled(true);
 			applyButton->setEnabled(f->isActive() && (f->minargs() <= 1 || settings->rpn_mode));
 			descriptionView->setHtml(QString::fromStdString(str));
@@ -449,6 +520,8 @@ void FunctionsDialog::selectedFunctionChanged(const QModelIndex &index, const QM
 	delButton->setEnabled(false);
 	deactivateButton->setEnabled(false);
 	applyButton->setEnabled(false);
+	favouriteButton->setEnabled(false);
+	favouriteButton->setChecked(false);
 	descriptionView->clear();
 	selected_item = NULL;
 }
@@ -490,7 +563,9 @@ void FunctionsDialog::updateFunctions() {
 	function_cats.parent = NULL;
 	bool has_inactive = false, has_uncat = false;
 	std::list<tree_struct>::iterator it;
+	QStandardItem *item_sel = NULL;
 
+	functionsView->selectionModel()->blockSignals(true);
 	sourceModel->clear();
 	sourceModel->setColumnCount(1);
 	sourceModel->setHorizontalHeaderItem(0, new QStandardItem(tr("Function")));
@@ -534,23 +609,36 @@ void FunctionsDialog::updateFunctions() {
 					cat_i_prev = cat_i + 1;
 					cat_i = cat.find("/", cat_i_prev);
 				}
-			} else if(!f->isLocal()) {
+			} else {
 				has_uncat = true;
 			}
 		}
-		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true)));
+		QStandardItem *item = new QStandardItem(QString::fromStdString(f->title(true, settings->printops.use_unicode_signs, &can_display_unicode_string_function, (void*) functionsView)));
 		item->setEditable(false);
 		item->setData(QVariant::fromValue((void*) f), Qt::UserRole);
 		sourceModel->appendRow(item);
-		if(f == selected_item) functionsView->selectionModel()->setCurrentIndex(functionsModel->mapFromSource(item->index()), QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+		if(f == selected_item) item_sel = item;
 	}
+	functionsView->selectionModel()->blockSignals(false);
 	sourceModel->sort(0);
 	function_cats.sort();
 
+	categoriesView->blockSignals(true);
 	categoriesView->clear();
+	categoriesView->blockSignals(false);
 	QTreeWidgetItem *iter, *iter2, *iter3;
 	QStringList l;
-	l << tr("All", "All functions"); l << "All";
+	l.clear(); l << tr("Favorites"); l << "Favorites";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "Favorites") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("User functions"); l << "User items";
+	iter = new QTreeWidgetItem(categoriesView, l);
+	if(selected_category == "User items") {
+		iter->setSelected(true);
+	}
+	l.clear(); l << tr("All", "All functions"); l << "All";
 	iter3 = new QTreeWidgetItem(categoriesView, l);
 	tree_struct *item, *item2;
 	function_cats.it = function_cats.items.begin();
@@ -599,11 +687,6 @@ void FunctionsDialog::updateFunctions() {
 			iter->setSelected(true);
 		}
 	}
-	l.clear(); l << tr("User functions"); l << "User items";
-	iter = new QTreeWidgetItem(iter3, l);
-	if(selected_category == "User items") {
-		iter->setSelected(true);
-	}
 	if(has_inactive) {
 		//add "Inactive" category if there are inactive functions
 		l.clear(); l << tr("Inactive"); l << "Inactive";
@@ -612,12 +695,24 @@ void FunctionsDialog::updateFunctions() {
 			iter->setSelected(true);
 		}
 	}
+	iter3->setExpanded(true);
 	if(categoriesView->selectedItems().isEmpty()) {
 		//if no category has been selected (previously selected has been renamed/deleted), select "All"
 		selected_category = "All";
-		iter3->setExpanded(true);
 		iter3->setSelected(true);
 	}
+	searchEdit->blockSignals(true);
+	searchEdit->clear();
+	searchEdit->blockSignals(false);
+	functionsModel->setFilter(selected_category);
+	QModelIndex index;
+	if(item_sel) index = functionsModel->mapFromSource(item_sel->index());
+	if(!index.isValid()) index = functionsModel->index(0, 0);
+	if(index.isValid()) {
+		functionsView->selectionModel()->setCurrentIndex(index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Clear);
+		functionsView->scrollTo(index);
+	}
+	selectedFunctionChanged(index, QModelIndex());
 }
 void FunctionsDialog::setSearch(const QString &str) {
 	searchEdit->setText(str);
