@@ -33,6 +33,13 @@
 #include "historyview.h"
 #include "qalculateqtsettings.h"
 
+bool qstring_has_nondigit(const QString &str) {
+	for(int i = 0; i < str.length(); i++) {
+		if(!str[i].isDigit()) return true;
+	}
+	return false;
+}
+
 QString unhtmlize(QString str) {
 	int i = 0, i2;
 	str.remove("\n");
@@ -51,6 +58,7 @@ QString unhtmlize(QString str) {
 				QString str2 = unhtmlize(str.mid(i2 + 1, i3 - i2 - 1));
 				if(str2.length() == 1 && str2[0] == '2') str.replace(i, i3 - i, SIGN_POWER_2);
 				else if(str2.length() == 1 && str2[0] == '3') str.replace(i, i3 - i, SIGN_POWER_3);
+				else if(str.length() == i3 + 7 && (str2.length() == 1 || !qstring_has_nondigit(str2))) str.replace(i, i3 - i, "^" + str2);
 				else str.replace(i, i3 - i, "^(" + str2 + ")");
 				continue;
 			}
@@ -92,6 +100,7 @@ QString unhtmlize(QString str) {
 					QString str2 = unhtmlize(str.mid(i + 5, i3 - i - 5));
 					if(str2.length() == 1 && str2[0] == '2') str.replace(i, i3 - i + 6, SIGN_POWER_2);
 					else if(str2.length() == 1 && str2[0] == '3') str.replace(i, i3 - i + 6, SIGN_POWER_3);
+					else if(str.length() == i3 + 6 && (str2.length() == 1 || !qstring_has_nondigit(str2))) str.replace(i, i3 - i + 6, "^" + str2);
 					else str.replace(i, i3 - i + 6, "^(" + str2 + ")");
 					continue;
 				}
@@ -122,11 +131,14 @@ QString unhtmlize(QString str) {
 	str.replace("&amp;", "&");
 	str.replace("&gt;", ">");
 	str.replace("&lt;", "<");
+	str.replace("&nbsp;", " ");
 	str.replace("&quot;", "\"");
 	return str;
 }
 
 HistoryView::HistoryView(QWidget *parent) : QTextEdit(parent), i_pos(0) {
+	i_pos = 0;
+	last_ans = 0;
 	setReadOnly(true);
 	QImage img1px(1, 1, QImage::Format_ARGB32);
 	img1px.fill(Qt::transparent);
@@ -197,22 +209,26 @@ void remove_top_border(QString &s_text) {
 }
 
 void HistoryView::loadInitial() {
+	s_text.clear();
+	i_pos = 0;
+	last_ans = 0;
+	last_ref = "";
 	if(!settings->v_expression.empty()) {
 		for(size_t i = 0; i < settings->v_expression.size(); i++) {
 			addResult(settings->v_result[i], settings->v_expression[i], true, false, QString(), NULL, true, i);
 		}
-		if(!s_text.isEmpty()) {
-			if((settings->color == 2 && (s_text.contains("color:#00") || s_text.contains("color:#58"))) || (settings->color != 2 && (s_text.contains("color:#FF") || s_text.contains("color:#AA")))) {
-				replace_colors(s_text);
-				for(size_t i = 0; i < settings->v_expression.size(); i++) {
-					replace_colors(settings->v_expression[i]);
-					for(size_t i2 = 0; i2 < settings->v_result[i].size(); i2++) {
-						replace_colors(settings->v_result[i][i2]);
-					}
+	}
+	if(!s_text.isEmpty()) {
+		if((settings->color == 2 && (s_text.contains("color:#00") || s_text.contains("color:#58"))) || (settings->color != 2 && (s_text.contains("color:#FF") || s_text.contains("color:#AA")))) {
+			replace_colors(s_text);
+			for(size_t i = 0; i < settings->v_expression.size(); i++) {
+				replace_colors(settings->v_expression[i]);
+				for(size_t i2 = 0; i2 < settings->v_result[i].size(); i2++) {
+					replace_colors(settings->v_result[i][i2]);
 				}
 			}
-			setHtml("<body color=\"" + textColor().name() + "\"><table width=\"100%\">" + s_text + "</table></body>");
 		}
+		setHtml("<body color=\"" + textColor().name() + "\"><table width=\"100%\">" + s_text + "</table></body>");
 	}
 }
 
