@@ -76,6 +76,8 @@ int main(int argc, char **argv) {
 	parser->addOption(tOption);
 	QCommandLineOption vOption(QStringList() << "v" << "verison", QApplication::tr("Display the application version"));
 	parser->addOption(vOption);
+	QCommandLineOption wOption(QStringList() << "w" << "workspace", QApplication::tr("Open workspace"), QApplication::tr("FILE"));
+	parser->addOption(wOption);
 	parser->addPositionalArgument("expression", QApplication::tr("Expression to calculate"), QApplication::tr("[EXPRESSION]"));
 	parser->addHelpOption();
 	parser->process(app);
@@ -91,7 +93,7 @@ int main(int argc, char **argv) {
 	if(settings->allow_multiple_instances < 1 && !parser->isSet(nOption) && !lockFile.tryLock(100)) {
 		if(lockFile.error() == QLockFile::LockFailedError) {
 			bool ami_changed = false;
-			if(settings->allow_multiple_instances < 0 && parser->value(fOption).isEmpty() && parser->positionalArguments().isEmpty()) {
+			if(settings->allow_multiple_instances < 0 && parser->value(fOption).isEmpty() && parser->value(wOption).isEmpty() && parser->positionalArguments().isEmpty()) {
 				settings->allow_multiple_instances = (QMessageBox::question(NULL, QString("Allow multiple instances?"), QApplication::tr("By default, only one instance (one main window) of %1 is allowed.\n\nIf multiple instances are opened simultaneously, only the definitions (variables, functions, etc.), mode, preferences, and history of the last closed window will be saved.\n\nDo you, despite this, want to change the default behavior and allow multiple simultaneous instances?").arg("Qalculate!"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes);
 				ami_changed = true;
 				if(settings->allow_multiple_instances) {
@@ -114,6 +116,9 @@ int main(int argc, char **argv) {
 					if(!parser->value(fOption).isEmpty()) {
 						command = "f";
 						command += parser->value(fOption);
+					} else if(!parser->value(wOption).isEmpty()) {
+						command = "w";
+						command += parser->value(wOption);
 					} else if(ami_changed) {
 						command = "-";
 					} else {
@@ -142,6 +147,8 @@ int main(int argc, char **argv) {
 
 	settings->loadPreferences();
 
+	if(!parser->value(wOption).isEmpty()) settings->loadWorkspace(parser->value(wOption).toLocal8Bit().data());
+
 	CALCULATOR->loadExchangeRates();
 
 	if(!CALCULATOR->loadGlobalDefinitions()) {
@@ -150,6 +157,15 @@ int main(int argc, char **argv) {
 		return 1;
 	}
 
+	if(settings->do_imaginary_j && CALCULATOR->getVariableById(VARIABLE_ID_I)->hasName("j") == 0) {
+		Variable *v_i = CALCULATOR->getVariableById(VARIABLE_ID_I);
+		ExpressionName ename = v_i->getName(1);
+		ename.name = "j";
+		ename.reference = false;
+		v_i->addName(ename, 1, true);
+		v_i->setChanged(false);
+	}
+
 	settings->f_answer->setCategory(CALCULATOR->getFunctionById(FUNCTION_ID_WARNING)->category());
 	settings->f_answer->setChanged(false);
 
@@ -159,7 +175,7 @@ int main(int argc, char **argv) {
 
 	QalculateWindow *win = new QalculateWindow();
 	if(parser->value(tOption).isEmpty()) {
-		win->updateWindowTitle();
+		win->updateWindowTitle(QString(), false, true);
 		title_modified = false;
 	} else {
 		win->setWindowTitle(parser->value(tOption));
