@@ -67,6 +67,8 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	combo->addItem(tr("Application name"), TITLE_APP);
 	combo->addItem(tr("Result"), TITLE_RESULT);
 	combo->addItem(tr("Application name + result"), TITLE_APP_RESULT);
+	combo->addItem(tr("Workspace"), TITLE_WORKSPACE);
+	combo->addItem(tr("Application name + workspace"), TITLE_APP_WORKSPACE);
 	combo->setCurrentIndex(combo->findData(settings->title_type));
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(titleChanged(int)));
 	l2->addWidget(combo, r, 1); r++;
@@ -130,6 +132,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	parseCombo = combo;
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(parsingModeChanged(int)));
 	l2->addWidget(combo, r, 1); r++;
+	BOX_G(tr("Simplified percentage calculation"), settings->simplified_percentage, simplifiedPercentageToggled(bool));
 	BOX_G(tr("Read precision"), settings->evalops.parse_options.read_precision != DONT_READ_PRECISION, readPrecisionToggled(bool));
 	BOX_G(tr("Limit implicit multiplication"), settings->evalops.parse_options.limit_implicit_multiplication, limitImplicitToggled(bool));
 	l2->addWidget(new QLabel(tr("Interval calculation:"), this), r, 0);
@@ -154,7 +157,6 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	decimalCommaBox = box;
 	BOX(tr("Ignore comma in numbers"), settings->evalops.parse_options.comma_as_separator, ignoreCommaToggled(bool)); ignoreCommaBox = box;
 	BOX(tr("Ignore dots in numbers"), settings->evalops.parse_options.dot_as_separator, ignoreDotToggled(bool)); ignoreDotBox = box;
-	BOX(tr("Round halfway numbers to even"), settings->printops.round_halfway_to_even, roundEvenToggled(bool));
 	BOX(tr("Indicate repeating decimals"), settings->printops.indicate_infinite_series, repeatingDecimalsToggled(bool));
 	if(CALCULATOR->getDecimalPoint() == COMMA) ignoreCommaBox->hide();
 	if(CALCULATOR->getDecimalPoint() == DOT) ignoreDotBox->hide();
@@ -181,6 +183,14 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	else combo->setCurrentIndex(combo->findData(settings->printops.interval_display));
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(intervalDisplayChanged(int)));
 	l2->addWidget(combo, r, 1); r++;
+	l2->addWidget(new QLabel(tr("Rounding:"), this), r, 0);
+	combo = new QComboBox(this);
+	combo->addItem(tr("Round halfway numbers away from zero"), 0);
+	combo->addItem(tr("Round halfway numbers to even"), 1);
+	combo->addItem(tr("Truncate all numbers"), 2);
+	combo->setCurrentIndex(combo->findData(settings->rounding_mode));
+	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(roundingChanged(int)));
+	l2->addWidget(combo, r, 1); r++;
 	l2->addWidget(new QLabel(tr("Complex number form:"), this), r, 0);
 	combo = new QComboBox(this);
 	combo->addItem(tr("Rectangular"), COMPLEX_NUMBER_FORM_RECTANGULAR);
@@ -322,6 +332,7 @@ void PreferencesDialog::imaginaryJToggled(bool b) {
 		v_i->setChanged(false);
 	}
 	emit expressionFormatUpdated(false);
+	emit symbolsUpdated();
 }
 void PreferencesDialog::decimalCommaToggled(bool b) {
 	settings->decimal_comma = b;
@@ -388,6 +399,10 @@ void PreferencesDialog::abbreviateNamesToggled(bool b) {
 	settings->printops.abbreviate_names = b;
 	emit resultFormatUpdated();
 }
+void PreferencesDialog::simplifiedPercentageToggled(bool b) {
+	settings->simplified_percentage = b;
+	emit expressionFormatUpdated(true);
+}
 void PreferencesDialog::readPrecisionToggled(bool b) {
 	if(b) settings->evalops.parse_options.read_precision = READ_PRECISION_WHEN_DECIMALS;
 	else settings->evalops.parse_options.read_precision = DONT_READ_PRECISION;
@@ -405,8 +420,10 @@ void PreferencesDialog::complexFormChanged(int i) {
 	settings->complex_angle_form = (settings->evalops.complex_number_form == COMPLEX_NUMBER_FORM_CIS);
 	emit expressionCalculationUpdated(0);
 }
-void PreferencesDialog::roundEvenToggled(bool b) {
-	settings->printops.round_halfway_to_even = b;
+void PreferencesDialog::roundingChanged(int i) {
+	settings->rounding_mode = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
+	settings->printops.custom_time_zone = (settings->rounding_mode == 2 ? -21586 : 0);
+	settings->printops.round_halfway_to_even = (settings->rounding_mode == 1);
 	emit resultFormatUpdated();
 }
 void PreferencesDialog::repeatingDecimalsToggled(bool b) {
@@ -536,17 +553,22 @@ void PreferencesDialog::darkModeToggled(bool b) {
 		if(styleCombo->currentIndex() == 0 || styleCombo->currentText().compare("windowsvista", Qt::CaseInsensitive) == 0) {
 			for(int i = 1; i < styleCombo->count(); i++) {
 				if(styleCombo->itemText(i).compare("Fusion", Qt::CaseInsensitive) == 0) {
+					int prev_style = settings->style;
 					styleCombo->setCurrentIndex(i);
+					settings->light_style = prev_style;
 					break;
 				}
 			}
 		}
+	} else if(settings->light_style != settings->style && styleCombo->currentText().compare("Fusion", Qt::CaseInsensitive) == 0) {
+		styleCombo->setCurrentIndex(settings->light_style < 0 ? 0 : settings->light_style + 1);
 	}
 #endif
 	settings->updatePalette();
 }
 void PreferencesDialog::styleChanged(int i) {
 	settings->style = i - 1;
+	settings->light_style = settings->style;
 	settings->updateStyle();
 }
 void PreferencesDialog::factorizeToggled(bool b) {
