@@ -56,9 +56,10 @@ class PreferencesDialog : public QDialog {
 		void exratesChanged(int);
 		void binaryPrefixesToggled(bool);
 		void readPrecisionToggled(bool);
+		void simplifiedPercentageToggled(bool);
 		void intervalCalculationChanged(int);
 		void complexFormChanged(int);
-		void roundEvenToggled(bool);
+		void roundingChanged(int);
 		void repeatingDecimalsToggled(bool);
 		void mixedUnitsToggled(bool);
 		void abbreviateNamesToggled(bool);
