@@ -44,7 +44,17 @@ class PeriodicTableDialog;
 class CalendarConversionDialog;
 class QTableWidget;
 class QMenu;
+class QTreeWidget;
+class QTreeWidgetItem;
+class QPushButton;
+class QListWidget;
+class QListWidgetItem;
+class QLineEdit;
+class QLabel;
+class QDialog;
+class QComboBox;
 struct FunctionDialog;
+struct keyboard_shortcut;
 
 class QalculateWindow : public QMainWindow {
 
@@ -57,7 +67,7 @@ class QalculateWindow : public QMainWindow {
 
 		void setCommandLineParser(QCommandLineParser*);
 		bool displayMessages();
-		bool updateWindowTitle(const QString &str = QString(), bool is_result = false);
+		bool updateWindowTitle(const QString &str = QString(), bool is_result = false, bool type_change = false);
 		void executeFromFile(const QString&);
 		void loadInitialHistory();
 
@@ -83,24 +93,31 @@ class QalculateWindow : public QMainWindow {
 		PlotDialog *plotDialog;
 		PeriodicTableDialog *periodicTableDialog;
 		CalendarConversionDialog *calendarConversionDialog;
+		QDialog *shortcutsDialog, *shortcutActionDialog;
+		QComboBox *shortcutActionValueEdit; QListWidget *shortcutActionList; QLabel *shortcutActionValueLabel;
+		QTreeWidget *shortcutList; QPushButton *addShortcutButton, *editShortcutButton, *removeShortcutButton, *shortcutActionOKButton;
 
 		KeypadWidget *keypad;
 		QDockWidget *keypadDock, *basesDock, *rpnDock;
 		QLabel *binEdit, *octEdit, *decEdit, *hexEdit;
 		QLabel *binLabel, *octLabel, *decLabel, *hexLabel;
 		QToolBar *tb;
-		QToolButton *menuAction, *modeAction;
-		QAction *toAction, *storeAction, *variablesAction, *functionsAction, *unitsAction, *plotAction, *fpAction, *calendarsAction, *percentageAction, *keypadAction, *basesAction, *customOutputBaseAction, *customInputBaseAction;
+		QToolButton *menuAction_t, *modeAction_t, *keypadAction_t;
+		QAction *toAction, *storeAction, *functionsAction_t, *unitsAction_t, *plotAction_t, *basesAction, *customOutputBaseAction, *customInputBaseAction, *newVariableAction, *newFunctionAction, *variablesAction, *functionsAction, *unitsAction, *datasetsAction, *plotAction, *fpAction, *calendarsAction, *percentageAction, *periodicTableAction, *exratesAction, *quitAction, *helpAction, *keypadAction, *rpnAction, *chainAction, *gKeypadAction, *pKeypadAction, *xKeypadAction, *cKeypadAction, *hideNumpadAction, *radAction, *degAction, *graAction, *normalAction, *sciAction, *engAction, *simpleAction;
 		QMenu *variablesMenu, *functionsMenu, *unitsMenu;
 		QAction *assumptionTypeActions[5], *assumptionSignActions[6];
+		QMenu *recentWSMenu;
+		QAction *recentWSSeparator, *openWSAction, *defaultWSAction, *saveWSAction, *saveWSAsAction;
+		QList<QAction*> recentWSAction;
 		QSpinBox *customOutputBaseEdit, *customInputBaseEdit;
 		QTimer *ecTimer, *rfTimer;
 		QFont saved_app_font;
 
 		QTableWidget *rpnView;
-		QAction *rpnUpAction, *rpnDownAction, *rpnSwapAction, *rpnCopyAction, *rpnLastxAction, *rpnDeleteAction, *rpnEditAction, *rpnClearAction;
+		QAction *rpnUpAction, *rpnDownAction, *rpnSwapAction, *rpnCopyAction, *rpnLastxAction, *rpnDeleteAction, *rpnClearAction;
 
 		bool send_event;
+		bool workspace_changed;
 
 		void calculateExpression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, std::string execute_str = std::string(), std::string str = std::string(), bool check_exrates = true);
 		void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, std::string transformation = "", bool do_stack = false, size_t stack_index = 0, bool register_moved = false, bool supress_dialog = false);
@@ -118,6 +135,12 @@ class QalculateWindow : public QMainWindow {
 		void RPNRegisterAdded(std::string, int = 0);
 		void RPNRegisterRemoved(int);
 		void RPNRegisterChanged(std::string, int);
+		void triggerShortcut(int, const std::string&);
+		void loadShortcuts();
+		bool editKeyboardShortcut(keyboard_shortcut*, keyboard_shortcut* = NULL, int = 0);
+		void loadWorkspace(const QString &filename);
+		void updateWSActions();
+		int askSaveWorkspace();
 
 	protected slots:
 
@@ -142,9 +165,13 @@ class QalculateWindow : public QMainWindow {
 		void onMPlusClicked();
 		void onMMinusClicked();
 		void onAnswerClicked();
+		void onBaseClicked(int, bool);
+		void onFactorizeClicked();
+		void onExpandClicked();
 		void onToActivated();
 		void onStoreActivated();
-		void onKeypadActivated(bool);
+		void keypadTypeActivated();
+		void hideNumpad(bool);
 		void onKeypadVisibilityChanged(bool);
 		void onBasesActivated(bool);
 		void onBasesVisibilityChanged(bool);
@@ -215,6 +242,24 @@ class QalculateWindow : public QMainWindow {
 		void updateFunctionsMenu();
 		void updateUnitsMenu();
 		void updateVariablesMenu();
+		void shortcutActivated();
+		void shortcutClicked(int, const QString&);
+		void keyboardShortcutAdded(keyboard_shortcut *ks);
+		void keyboardShortcutRemoved(keyboard_shortcut *ks);
+		void addShortcutClicked();
+		void editShortcutClicked();
+		void removeShortcutClicked();
+		void shortcutActionOKClicked();
+		void updateShortcutActionOK();
+		void currentShortcutActionChanged(QListWidgetItem*, QListWidgetItem*);
+		void currentShortcutChanged(QTreeWidgetItem*, QTreeWidgetItem*);
+		void shortcutDoubleClicked(QTreeWidgetItem*, int);
+		void saveWorkspace();
+		void saveWorkspaceAs();
+		void openWorkspace();
+		void openDefaultWorkspace();
+		void openRecentWorkspace();
+
 
 	public slots:
 
@@ -240,6 +285,7 @@ class QalculateWindow : public QMainWindow {
 		void convertToUnit(Unit*);
 		void importCSV();
 		void exportCSV();
+		void editKeyboardShortcuts();
 		void editPreferences();
 		void onDatasetsChanged();
 		void onFunctionsChanged();
